#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include <util/delay.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "timeout.h" // defines F_CPU
#include "ili9163/ili9163lcd.h"
#include "colors.h"

ILI9163 lcd1; 
ILI9163 lcd2;
DigitalPin	pinSCK(&PORTB, PB7), pinSDA(&PORTB, PB5), pinA0(&PORTB, PB2), pinRESET(&PORTB, PB3), pinCS1(&PORTB, PB0), pinCS2(&PORTB, PB1);

void CS1_SELECT() {
	PORTB &= ~(1 << PB0);
	PORTB |= (1 << PB1);
}

void CS2_SELECT() {
	PORTB &= ~(1 << PB1);
	PORTB |= (1 << PB0);
}

void CS1_DESELECT() {
	PORTB |= (1 << PB0);
}

void CS2_DESELECT() {
	PORTB |= (1 << PB1);
}
int count = 0;
unsigned int rand(unsigned int startNumber,unsigned int endNumber)
{
	unsigned int lfsr = 0xACE1u;
	unsigned int bit, t = 0;
    int *p = (int*)malloc(sizeof(int));
    t = t^(int)p;
    bit  = ((lfsr >> 0) ^ (lfsr >> 2) ^ (lfsr >> 3) ^ (lfsr >> 5) ) & 1;
    lfsr = ((bit<<15) | (lfsr>>1) | t)%endNumber;
    while(lfsr<startNumber){
        lfsr = lfsr + endNumber - startNumber;
    }
    return lfsr;
}

void drawCard(int topx, int topy, int xdim, int ydim, int lcd, char value, int type){
	uint16_t col;
	if (type == 3){
		col = RED;
	} else if (type == 4){
		col = BLACK;
	} else if (type == 5){
		col = RED;
	} else if (type == 6){
		col = BLACK;
	}
	
	if (lcd == 1){
		CS1_SELECT();
		lcd1.drawLine(topx, topy, topx + xdim, topy, col);
		lcd1.drawLine(topx, topy, topx, topy + ydim, col);
		lcd1.drawLine(topx + xdim, topy, topx + xdim, topy + ydim, col);
		lcd1.drawLine(topx, topy + ydim, topx + xdim, topy + ydim, col);
		lcd1.drawChar(value, topx + 1, topy-2+ydim/2, col, GREEN);
		lcd1.drawChar((char)type, topx + 8, topy-2+ydim/2, col, GREEN);

		
	} else {
		CS2_SELECT();
		lcd2.drawLine(topx, topy, topx + xdim, topy, col);
		lcd2.drawLine(topx, topy, topx, topy + ydim, col);
		lcd2.drawLine(topx + xdim, topy, topx + xdim, topy + ydim, col);
		lcd2.drawLine(topx, topy + ydim, topx + xdim, topy + ydim, col);
		lcd2.drawChar(value, topx + 1, topy-2+ydim/2, col, GREEN); 
		lcd2.drawChar((char)type, topx + 8, topy-2+ydim/2, col, GREEN);
	}
}

char getValue(int c){
	if (c==0)
		return 'A';
	if (c==1)
		return '2';
	if (c==2)
		return '3';
	if (c==3)
		return '4';
	if (c==4)
		return '5';
	if (c==5)
		return '6';
	if (c==6)
		return '7';
	if (c==7)
		return '8';
	if (c==8)
		return '9';
	if (c==9)
		return 'Z';
	if (c==10)
		return 'J';
	if (c==11)
		return 'Q';
	if (c==12)
		return 'K';	
}

int main (void)
{
	sei();
	lcd1.setup(&pinSCK, &pinSDA, &pinA0, &pinRESET, &pinCS1);
	lcd2.setup(&pinSCK, &pinSDA, &pinA0, &pinRESET, &pinCS2);

	lcd1.init(ILI9163::BOTTOM);
	//CS1_SELECT();
	lcd1.clear(WHITE);
	int i, j;
	int p1_cards[5];
	int p2_cards[5];
	int cards[13][4];
	for (i = 0; i < 13; i++){
		for(j = 0; j < 4; j++){
				cards[i][j] = 0;
		}
	}
	//CS2_SELECT();
	//lcd2.clear(WHITE);
	/*CS1_SELECT();
	//lcd1.drawCircle(80, 100, 10, RED);
	//lcd1.drawCircle(40, 100, 10, RED);
	//lcd1.drawLine(50, 100, 50, 40, RED);

	CS2_SELECT();
	lcd2.clear(YELLOW);

	CS1_SELECT();
	//lcd1.clear(BLUE);

	CS2_SELECT();
	//lcd2.drawCircle(80, 100, 10, RED);
	lcd2.drawCircle(40, 100, 10, RED);
	lcd2.drawLine(50, 100, 50, 40, RED);
	lcd2.drawLine(70, 100, 70, 40, RED);
	lcd2.drawCircle(60, 40, 10, RED);
	lcd2.drawLine(60, 40, 60, 30, RED);
	lcd2.drawRectFilled(51, 40, 69, 50, YELLOW);
*/	

	int pressedA = 0, pressedB = 0;

	lcd1.drawString("PRESS START", 35, 55, RED, WHITE);
	
	CS2_SELECT();
	lcd2.drawString("Player 1", 40, 110, RED, WHITE);
	CS1_SELECT();
	lcd1.drawString("Player 2", 40, 110, RED, WHITE);
	
	
	
	while (pressedA + pressedB < 2){
		//LCD 2 - CEL LEGAT LA PORTA
		//setez pin ca intrare
		DDRC &= ~(1 << PC0);
		//activez rezitenta de pull-up
		PORTC |= (1 << PC0);
		
		_delay_ms(25);
		//daca e apasat
		if ((PINC & (1 << PC0)) == 0) {
			pressedA = 1;
		}
		
		//LCD 1 - CEL LEGAT LA PORTD
		//setez pin ca intrare
		DDRD &= ~(1 << PD0);
		//activez rezitenta de pull-up
		PORTD |= (1 << PD0);
		
		_delay_ms(25);
			//daca e apasat
		if ((PIND & (1 << PD0)) == 0) {
			pressedB = 1;
		}
		
	
	}

	//DRAW INITIAL CARDS
	pressedA = 0;
	pressedB = 0;
	
	CS1_SELECT();
	lcd1.clear(GREEN);
	lcd1.drawString("Wait for Player 1", 10, 110, RED, GREEN);
	int card1p1=0, card2p1=0, card3p1=0, card4p1=0, card5p1=0;
	int card1p2=0, card2p2=0, card3p2=0, card4p2=0, card5p2=0;
	CS2_SELECT();
	lcd2.clear(GREEN);
	/* random int between 0 and 13 */
	int c = rand(0, 12);
	int cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	char carte = getValue(c);
	drawCard(10, 90, 20, 30, 2, carte, cul);	
	
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(32, 90, 20, 30, 2, carte, cul);
	
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(54, 90, 20, 30, 2, carte, cul);
	
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(76, 90, 20, 30, 2, carte, cul);
	
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(98, 90, 20, 30, 2, carte, cul);
	int selectedA = 0;
	while (pressedA == 0){
		//LCD 2 - CEL LEGAT LA PORTA
		//setez pin ca intrare
		DDRC &= ~(1 << PC0);
		//activez rezitenta de pull-up
		PORTC |= (1 << PC0);
		
		_delay_ms(25);
		//daca e apasat
		if ((PINC & (1 << PC0)) == 0) {
			pressedA = 1;
		}
		
		//setez pin ca intrare
		DDRC &= ~(1 << PC1);
		//activez rezitenta de pull-up
		PORTC |= (1 << PC1);
		
		_delay_ms(25);
			//daca e apasat
		if ((PINC & (1 << PC1)) == 0) {
			if (card1p1 == 0 && selectedA < 3){
				card1p1 = 1;
				selectedA++;
				lcd2.drawCircle(20, 85, 3, BLACK);
			} else
			if (card1p1 == 1){
				card1p1 = 0;
				selectedA--;
				lcd2.drawCircle(20, 85, 3, GREEN);
			}
		}
		
		//setez pin ca intrare
		DDRA &= ~(1 << PA2);
		//activez rezitenta de pull-up
		PORTA |= (1 << PA2);
		
		_delay_ms(25);
			//daca e apasat
		if ((PINA & (1 << PA2)) == 0) {
			if (card2p1 == 0 && selectedA < 3){
				card2p1 = 1;
				selectedA++;
				lcd2.drawCircle(42, 85, 3, BLACK);
			} else
			if (card2p1 == 1){
				lcd2.drawCircle(42, 85, 3, GREEN);
				card2p1 = 0;
				selectedA--;
			}
		}

		//setez pin ca intrare
		DDRA &= ~(1 << PA3);
		//activez rezitenta de pull-up
		PORTA |= (1 << PA3);
		
		_delay_ms(25);
			//daca e apasat
		if ((PINA & (1 << PA3)) == 0) {
			if (card3p1 == 0 && selectedA < 3){
				card3p1 = 1;
				selectedA++;
				lcd2.drawCircle(64, 85, 3, BLACK);
			} else
			if (card3p1 == 1){
				card3p1 = 0;
				selectedA--;
				lcd2.drawCircle(64, 85, 3, GREEN);
			}
		}
		
		//setez pin ca intrare
		DDRA &= ~(1 << PA4);
		//activez rezitenta de pull-up
		PORTA |= (1 << PA4);
		
		_delay_ms(25);
		//daca e apasat
		if ((PINA & (1 << PA4)) == 0) {
			if (card4p1 == 0 && selectedA < 3){
				card4p1 = 1;
				selectedA++;
				lcd2.drawCircle(86, 85, 3, BLACK);
			} else
			if (card4p1 == 1){
				card4p1 = 0;
				selectedA--;
				lcd2.drawCircle(86, 85, 3, GREEN);
			}
		}
		//setez pin ca intrare
		DDRA &= ~(1 << PA5);
		//activez rezitenta de pull-up
		PORTA |= (1 << PA5);
		
		_delay_ms(25);
			//daca e apasat
		if ((PINA & (1 << PA5)) == 0) {
			if (card5p1 == 0 && selectedA < 3){
				card5p1 = 1;
				selectedA++;
				lcd2.drawCircle(109, 85, 3, BLACK);
			} else
			if (card5p1 == 1){
				card5p1 = 0;
				selectedA--;
				lcd2.drawCircle(109, 85, 3, GREEN);
			}
		}
	}
	pressedA = 0;
	lcd2.clear(GREEN);
	lcd2.drawString("Wait for Player 2", 10, 110, RED, GREEN);
	CS1_SELECT();
	lcd2.clear(GREEN);
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(10, 90, 20, 30, 1, carte, cul);	
	
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(32, 90, 20, 30, 1, carte, cul);
	
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(54, 90, 20, 30, 1, carte, cul);
	
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(76, 90, 20, 30, 1, carte, cul);
	
	c = rand(0, 12);
	cul = rand(3, 6);
	while (cards[c][cul] == 1){
		c = rand(0, 12);
		cul = rand(3, 6);
		c = 0; 
		cul = 6;
	}
	cards[c][cul] = 1;
	carte = getValue(c);
	drawCard(98, 90, 20, 30, 1, carte, cul);	
	
	int selected = 0;
	while (pressedB == 0){
		//LCD 2 - CEL LEGAT LA PORTA
		//setez pin ca intrare
		DDRD &= ~(1 << PD0);
		//activez rezitenta de pull-up
		PORTD |= (1 << PD0);
		
		_delay_ms(25);
		//daca e apasat
		if ((PIND & (1 << PD0)) == 0) {
			pressedB = 1;
		}
		
		//setez pin ca intrare
		DDRD &= ~(1 << PD1);
		//activez rezitenta de pull-up
		PORTD |= (1 << PD1);
		
		_delay_ms(25);
			//daca e apasat
		if ((PIND & (1 << PD1)) == 0) {
			if (card1p2 == 0 && selected < 3){
				selected++;
				card1p2 = 1;
				lcd1.drawCircle(20, 85, 3, BLACK);
			} else
			if (card1p2 == 1){		
				selected--;
				card1p2 = 0;
				lcd1.drawCircle(20, 85, 3, GREEN);
			}
		}
		
		//setez pin ca intrare
		DDRD &= ~(1 << PD6);
		//activez rezitenta de pull-up
		PORTD |= (1 << PD6);
		
		_delay_ms(25);
			//daca e apasat
		if ((PIND & (1 << PD6)) == 0) {
			if (card2p2 == 0 && selected < 3){
				card2p2 = 1;
				selected++;
				lcd1.drawCircle(42, 85, 3, BLACK);
			} else
			if (card2p2 == 1){
				lcd1.drawCircle(42, 85, 3, GREEN);
				card2p2 = 0;
				selected--;
			}
		}

		//setez pin ca intrare
		DDRA &= ~(1 << PA1);
		//activez rezitenta de pull-up
		PORTA |= (1 << PA1);
		
		_delay_ms(25);
			//daca e apasat
		if ((PINA & (1 << PA1)) == 0) {
			if (card3p2 == 0 && selected < 3){
				card3p2 = 1;
				selected++;
				lcd1.drawCircle(64, 85, 3, BLACK);
			} else
			if (card3p2 == 1){
				selected--;
				card3p2 = 0;
				lcd1.drawCircle(64, 85, 3, GREEN);
			}
		}
		
		//setez pin ca intrare
		DDRD &= ~(1 << PD4);
		//activez rezitenta de pull-up
		PORTD |= (1 << PD4);
		
		_delay_ms(25);
		//daca e apasat
		if ((PIND & (1 << PD4)) == 0) {
			if (card4p2 == 0 && selected < 3){
				card4p2 = 1;
				selected++;
				lcd1.drawCircle(86, 85, 3, BLACK);
			} else
			if (card4p2 == 1){
				selected--;
				card4p2 = 0;
				lcd1.drawCircle(86, 85, 3, GREEN);
			}
		}
		//setez pin ca intrare
		DDRD &= ~(1 << PD5);
		//activez rezitenta de pull-up
		PORTD |= (1 << PD5);
		
		_delay_ms(25);
			//daca e apasat
		if ((PIND & (1 << PD5)) == 0) {
			if (card5p2 == 0 && selected < 3){
				card5p2 = 1;
				selected++;
				lcd1.drawCircle(109, 85, 3, BLACK);
			} else
			if (card5p2 == 1){
				card5p2 = 0;
				selected--;
				lcd1.drawCircle(109, 85, 3, GREEN);
			}
		}
	}
	
	lcd1.clear(RED);
	
	CS2_SELECT();
	lcd2.clear(RED);
	/*//setez pin ca intrare
	DDRC &= ~(1 << PC0);
	//activez rezienta de pull-up
	PORTC |= (1 << PC0);
	
	_delay_ms(25);
	pressed = 0;
	while (pressed == 0) {
		//daca e apasat
		if ((PINC & (1 << PC0)) == 0) {
			pressed = 1;
		}
	}
	*/

	/*
	
	
	
	*/
	
	return 0;
}
